﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Primeworks.Bluetooth
{
	/// <summary>
	/// The BluetoothClient class provides all the Bluetooth services for the consuming application.
	/// </summary>
	public class BluetoothClient : System.IDisposable
	{
		private BluetoothStack				m_stack			= null;
		private BluetoothDeviceList			m_devices		= new BluetoothDeviceList();
		private DeviceFoundEventHandler		m_deviceFound	= null;
		private bool						m_bDisposed		= false;

		/// <summary>
		/// Initializes an instance of the BluetoothClient class.
		/// </summary>
		public BluetoothClient()
		{
			// Check if Widcomm is present
			if(Primeworks.Bluetooth.Widcomm.WidcommStack.Version != 0)
			{
				m_stack = new Primeworks.Bluetooth.Widcomm.WidcommStack();
			}
			else
			{
				m_stack = new Primeworks.Bluetooth.Microsoft.MicrosoftStack();
			}
			m_stack.InquiryStateChanged += new InquiryEvent(m_stack_InquiryStateChanged);
		}

		~BluetoothClient()
		{
			Dispose(false);
		}

		/// <summary>
		/// This event is raised when a device is found.
		/// </summary>
		/// <remarks>
		/// When device discovery ends this event is raised with a null device reference and
		/// the NoMoreDevices property set to <b>true</b>.
		/// </remarks>
		public event DeviceFoundEventHandler DeviceFound
		{
			add { m_deviceFound += value; }
			remove { m_deviceFound -= value; }
		}

		private void OnDeviceFound(BluetoothDevice device, bool bEnded)
		{
			if(m_deviceFound != null)
			{
				DeviceDiscoveryCancelEventArgs e = new DeviceDiscoveryCancelEventArgs(device, bEnded);

				e.Cancel = false;
				m_deviceFound(this, e);

				// Stop the inquiry if the user told us so.
				if(e.Cancel)
					m_stack.StopInquiry();
			}
		}

		private void m_stack_InquiryStateChanged(object sender, InquiryEventArgs e)
		{
			if(e.State == InquiryState.DeviceFound)
			{
				m_devices.Add(e.Device);

				OnDeviceFound(e.Device, false);
			}
			else
			{
				OnDeviceFound(null, true);
			}
		}

		/// <summary>
		/// Starts the asynchronous process of device discovery.
		/// </summary>
		/// <returns>
		/// <b>true</b> if the device discovery process was successfully started.
		/// </returns>
		public bool BeginDeviceDiscovery()
		{
			m_devices.Clear();

			return m_stack.StartInquiry();
		}

		/// <summary>
		/// Stops the device discovery process.
		/// </summary>
		public void StopDeviceDiscovery()
		{
			m_stack.StopInquiry();
		}

		/// <summary>
		/// Gets the list of discovered devices.
		/// </summary>
		public BluetoothDeviceList Devices
		{
			get { return m_devices; }
		}

		/// <summary>
		/// Gets the instantiated stack object.
		/// </summary>
		public BluetoothStack Stack
		{
			get { return m_stack; }
		}

		protected void Dispose(bool bDisposing)
		{
			if(m_bDisposed)
				return;

			if(bDisposing)
				m_stack.Dispose();
		}

		#region IDisposable Members

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
