﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Runtime.InteropServices;


namespace Primeworks.Bluetooth
{
	public class BluetoothDevice
	{
		private BluetoothStack				m_stack			= null;
		private string						m_strName		= "";
		private long						m_nAddress		= 0;
		private int							m_nClass		= 0;
		private bool						m_bConnected	= false;
		private ServiceDiscoveryEndedEvent	m_discovEvent	= null;
		private BluetoothServiceList		m_services		= new BluetoothServiceList();

		public BluetoothDevice(BluetoothStack stack, IntPtr pDevInfo)
		{
			BLUETOOTH_DEVICE_INFO btdi;

			btdi = (BLUETOOTH_DEVICE_INFO)Marshal.PtrToStructure(pDevInfo, typeof(BLUETOOTH_DEVICE_INFO));

			m_strName		= btdi.szName;
			m_nAddress		= btdi.Address;
			m_nClass		= btdi.ulClassOfDevice;
			m_bConnected	= btdi.fConnected != 0;

			m_stack			= stack;

			m_stack.ServiceDiscoveryEnded += new ServiceDiscoveryEndedEvent(m_stack_ServiceDiscoveryEnded);
		}

		private void m_stack_ServiceDiscoveryEnded(object sender, ServiceDiscoveryEventArgs e)
		{
			ServiceDiscovery discovery = m_stack.GetServiceDiscovery(m_nAddress);

			m_services = discovery.Services;

			if(m_discovEvent != null)
				m_discovEvent(this, e);
		}

		/// <summary>
		/// Gets the device name.
		/// </summary>
		public string Name
		{
			get { return m_strName; }
		}

		/// <summary>
		/// Gets the device class.
		/// </summary>
		public int Class
		{
			get { return m_nClass; }
		}

		/// <summary>
		/// Gets the device address as an array of 6 bytes.
		/// </summary>
		public byte[] AddressAsBytes
		{
			get
			{
				byte[] addr = new byte[6];

				addr[5] = (byte)(m_nAddress         & (long)0xff);
				addr[4] = (byte)((m_nAddress >>  8) & (long)0xff);
				addr[3] = (byte)((m_nAddress >> 16) & (long)0xff);
				addr[2] = (byte)((m_nAddress >> 24) & (long)0xff);
				addr[1] = (byte)((m_nAddress >> 32) & (long)0xff);
				addr[0] = (byte)((m_nAddress >> 40) & (long)0xff);

				return addr;
			}
		}

		/// <summary>
		/// Gets the device address as an Int64 value.
		/// </summary>
		public long Address
		{
			get { return m_nAddress; }
		}

		/// <summary>
		/// Gets the Bluetooth services available on the device.
		/// </summary>
		public BluetoothServiceList Services
		{
			get { return m_services; }
		}

		/// <summary>
		/// Begins the service discovery process.
		/// </summary>
		/// <returns>
		/// <b>true</b> if the service discovery process was successfully started.
		/// </returns>
		public bool BeginServiceDiscovery()
		{
			if(m_stack.BeginServiceDiscovery(m_nAddress))
			{
				m_services.Clear();
				return true;
			}
			return false;
		}

		public event ServiceDiscoveryEndedEvent ServiceDiscoveryEnded
		{
			add { m_discovEvent += value; }
			remove { m_discovEvent -= value; }
		}
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct SYSTEMTIME
	{
		public short wYear;
		public short wMonth;
		public short wDayOfWeek;
		public short wDay;
		public short wHour;
		public short wMinute;
		public short wSecond;
		public short wMilliseconds;
	};

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	public struct BLUETOOTH_DEVICE_INFO
	{
		public int dwSize;
		public long Address;
		public int ulClassOfDevice;
		public int fConnected;
		public int fRemembered;
		public int fAuthenticated;
		public SYSTEMTIME stLastSeen;
		public SYSTEMTIME stLastUsed;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 248)]
		public string szName;
	};
}
